`timescale 1ns / 1ps

module ALU(RegA, Inst, RegB, op, srcB, ALUout, zero, neg);

  input	[31:0] RegA;
  input	[31:0] Inst;
  input	[31:0] RegB;
  input	[5:0] op; 
  input	 srcB;
  output	[31:0] ALUout;
  output    zero, neg;

  wire [31:0] A;
  wire [31:0] B;
  reg  [31:0] result;
  reg  zero;
  reg  neg;

  assign A = RegA;

  // select the source for the B operand to the ALU (output B of register file, or
  // an immediate value stored in the instruction - sign-extended
  assign B = (srcB === 1'b0) ? RegB : {{16{Inst[15]}},Inst[15:0]};  // 16-bit immediate value with sign extension  
 
  // perform the ALU operation and set flags if the result is zero or negative

  always @(A or B or op) begin
    case (op)
      6'b000001: result = A + B;
      6'b000010: result = A - B;
      6'b000100: result = A & B;
      6'b001000: result = A | B;
      6'b010000: result = (A < B)? 1 : 0;  // Set on less than. Enables all branches using BEQ
      default:   result = 32'hxxxxxxxx;
    endcase
    zero = (result == 32'h00000000);
    neg  = result[31];    
  end

  // assign the computed result to the ALU output
  
  assign ALUout = result;

endmodule
