`timescale 1ns / 1ps

module PC(PCSel,Inst, reset, clk, PC);

  input		[31:0] Inst;
  input		reset, clk;
  input 	[1:0] PCSel;
  output  	[31:0] PC;

  reg [31:0] PC;
  reg [31:0] src; 
  
  wire [31:0] offset;
   
  // BEQ (and others) set PC to a value relative to the next PC. To allow
  // for loops, need ability for a negative offset, so sign extend the
  // offset provided as bits 15:0 of the instruction
  
  assign offset = {{16{Inst[15]}},Inst[15:0]}; // sign extend the offset
  
  // There are 3 possible sources for PC
  // 0.  PC = PC + 1		  (Move to next Instruction)
  // 1.  PC = PC + 1 + Offset (Conditional Branch)
  // 2,3.  PC = PC[31:26],Absolute ( J instruction)
  
  always @(PCSel or PC or Inst or reset) begin
	  casex(PCSel)
		  2'b00: src = PC + 1;
		  2'b01: src = PC + 1 + offset;
		  2'b1?: src = {PC[31:26], Inst[25:0]};	// take 6 top bits of PC, 26 bits from Inst
	  endcase 
	  src = (reset) ? 32'h00000000 : src;
  end

  // update the value of PC at the rising clock edge
  always @(posedge clk) begin
	  PC = src;
  end

endmodule
