`timescale 1ns / 1ps

module RegFile(WriteData, Inst, regWrite, wrRegSel, RegA, RegB, clk);

  input	[31:0] WriteData;
  input	[31:0] Inst;
  input	regWrite, wrRegSel;
  output    [31:0] RegA;
  output    [31:0] RegB;
  input	clk;

  wire [4:0]  rs, rt, rd, wrReg;


  reg [31:0] RegFile[0:31];
  wire [31:0] RegA, RegB;

  // initialize register 31 to be a zero - we'll use this as a constant
  
  initial begin
    RegFile[31] = 0;
  end

  // pick out the indices of the three possible registers involved in an instruction 
  
  assign rs = Inst[25:21];
  assign rt = Inst[20:16];
  assign rd = Inst[15:11];
  
  // decide which register is the one that might be written to (depends on instruction)
	  
  assign wrReg = wrRegSel ? rd : rt;   
  
  // do two reads and, optionally, one write with the register file
  // read two registers and send them to the ALU
  assign 	RegA = RegFile[rs];
  assign	RegB = RegFile[rt];
    
  // write into a register (but not the register storing our constant 0)	
  always @(posedge clk) begin
    if (regWrite && (wrReg != 31)) begin
      RegFile[wrReg] = WriteData;
    end
  end

endmodule
